﻿Imports System.Net
Imports System.Xml
Imports System.Text

Public NotInheritable Class BingTranslationActions
    Private Const AppId As String = "E6229FE0535A7ED00FAD685A301D0A767FBD2821"

    Private Sub New()
    End Sub
    Public Shared Function TranslationRequest(ByVal [from] As String, ByVal [to] As String, ByVal word As String) As String
        Dim request As HttpWebRequest = BuildRequest([from], [to], word)
        Try
            Return DisplayResponse(CType(request.GetResponse, HttpWebResponse))
        Catch ex As WebException
            'todo: handle exception
        End Try
        Throw New Exception(String.Format("There was an error processing the translation request of '{0}' from {1} to {2}", word, [from], [to]))
    End Function

    Private Shared Function BuildRequest(ByVal [from] As String, ByVal [to] As String, ByVal word As String) As HttpWebRequest
        ' Common request fields (required)
        ' Common request fields (optional)
        ' SourceType-specific request fields (required)
        Dim requestString As String = "http://api.bing.net/xml.aspx?" & "AppId=" & AppId & "&Query=" & word & "&Sources=Translation" & "&Version=2.2" & "&Translation.SourceLanguage=" & [from] & "&Translation.TargetLanguage=" & [to]

        ' Create and initialize the request.
        Dim request As HttpWebRequest = CType(HttpWebRequest.Create(requestString), HttpWebRequest)

        Return request
    End Function

    Private Shared Function DisplayResponse(ByVal response As HttpWebResponse) As String
        ' Load the response into an XmlDocument.
        Dim document As New XmlDocument
        document.Load(response.GetResponseStream)

        ' Add the default namespace to the namespace manager.
        Dim nsmgr As New XmlNamespaceManager(document.NameTable)
        nsmgr.AddNamespace("api", "http://schemas.microsoft.com/LiveSearch/2008/04/XML/element")

        Dim errors As XmlNodeList = document.DocumentElement.SelectNodes("./api:Errors/api:Error", nsmgr)

        If errors.Count > 0 Then
            ' There are errors in the response. Display error details.
            Return DisplayErrors(errors)
        Else
            ' There were no errors in the response. Display the
            ' Translation results.
            Return DisplayResults(document.DocumentElement, nsmgr)
        End If
    End Function

    Private Shared Function DisplayResults(ByVal root As XmlNode, ByVal nsmgr As XmlNamespaceManager) As String
        Dim str As New StringBuilder
        str.AppendLine("Bing API Version: " & root.SelectSingleNode("./@Version", nsmgr).InnerText)
        str.AppendLine("Search terms: " & root.SelectSingleNode("./api:Query/api:SearchTerms", nsmgr).InnerText)
        str.AppendLine()
        ' Add the Translation SourceType namespace to the namespace manager.
        nsmgr.AddNamespace("tra", "http://schemas.microsoft.com/LiveSearch/2008/04/XML/translation")

        Dim results As XmlNodeList = root.SelectNodes("./tra:Translation/tra:Results/tra:TranslationResult", nsmgr)

        ' Display the Translation results.
        For Each result In results
            str.AppendLine(result.SelectSingleNode("./tra:TranslatedTerm", nsmgr).InnerText)
        Next result
        Return str.ToString
    End Function

    Private Shared Function DisplayErrors(ByVal errors As XmlNodeList) As String
        Dim str As New StringBuilder
        ' Iterate over the list of errors and display error details.
        str.AppendLine("The following errors occurred while trying to translate:")
        str.AppendLine()
        For Each [error] In errors
            For Each detail In [error].ChildNodes
                str.AppendLine(detail.Name & ": " & detail.InnerText)
            Next detail
        Next [error]
        Return str.ToString
    End Function
End Class

