﻿Imports System.Net
Imports System.Xml
Imports System.Text

Public NotInheritable Class NewsActions
    Private Const AppId As String = "E6229FE0535A7ED00FAD685A301D0A767FBD2821"

    Private Sub New()
    End Sub

    Public Shared Function GetTheNews() As String
        Dim request As HttpWebRequest = BuildRequest()
        Try
            Return DisplayResponse(CType(request.GetResponse, HttpWebResponse))
        Catch ex As WebException
            'todo: handle exception
        End Try
        Throw New Exception("There was an error processing the news request.")
    End Function

    Private Shared Function BuildRequest() As HttpWebRequest
        'Common request fields (required) Common request fields (optional) News-specific request fields (optional) The following request fields are mutually exclusive.Uncomment the line corresponding to the request field you wish to use. & "&News.LocationOverride=US.VA"
        ''& "&News.Category=rt_Political"
        Dim requestString As String = "http://api.bing.net/xml.aspx?" & "AppId=" & AppId & "&Query=" & "&Sources=News" & "&Version=2.0" & "&Market=en-us" & "&Options=EnableHighlighting" & "&News.Offset=0" & "&News.SortBy=Relevance" 'msn moneycentral"
        ' Create and initialize the request.
        Dim request As HttpWebRequest = CType(HttpWebRequest.Create(requestString), HttpWebRequest)

        Return request
    End Function

    Private Shared Function DisplayResponse(ByVal response As HttpWebResponse) As String
        ' Load the response into an XmlDocument.
        Dim document As New XmlDocument
        document.Load(response.GetResponseStream)

        ' Add the default namespace to the namespace manager.
        Dim nsmgr As New XmlNamespaceManager(document.NameTable)
        nsmgr.AddNamespace("api", "http://schemas.microsoft.com/LiveSearch/2008/04/XML/element")

        Dim errors As XmlNodeList = document.DocumentElement.SelectNodes("./api:Errors/api:Error", nsmgr)

        If errors.Count > 0 Then
            ' There are errors in the response. Display error details.
            Return DisplayErrors(errors)
        Else
            ' There were no errors in the response. Display the
            ' News results.
            Return DisplayResults(document.DocumentElement, nsmgr)
        End If
    End Function

    Private Shared Function DisplayResults(ByVal root As XmlNode, ByVal nsmgr As XmlNamespaceManager) As String
        'Dim doc As XDocument = XDocument.Load(New XmlNodeReader(root))
        'Dim newsnmsp As XNamespace = "http://schemas.microsoft.com/LiveSearch/2008/04/XML/news"
        'Dim stories = From newsArt In doc.Elements(newsnmsp + "NewsArticle")
        '              Select
        Dim str As New StringBuilder
        ' Add the News SourceType namespace to the namespace manager.
        nsmgr.AddNamespace("news", "http://schemas.microsoft.com/LiveSearch/2008/04/XML/news")

        Dim news As XmlNode = root.SelectSingleNode("./news:News", nsmgr)
        Dim results As XmlNodeList = news.SelectNodes("./news:Results/news:NewsResult", nsmgr)

        Dim version As String = root.SelectSingleNode("./@Version", nsmgr).InnerText
        Dim searchTerms As String = root.SelectSingleNode("./api:Query/api:SearchTerms", nsmgr).InnerText
        str.AppendLine("Bing API Version " & version)
        str.AppendLine("News results for " & searchTerms)
        str.AppendLine()

        ' Display the News results.
        Dim builder As New System.Text.StringBuilder
        For Each result In results
            builder.AppendLine(result.SelectSingleNode("./news:Title", nsmgr).InnerText)
            builder.AppendLine(result.SelectSingleNode("./news:Url", nsmgr).InnerText)
            builder.AppendLine(result.SelectSingleNode("./news:Source", nsmgr).InnerText)
        Next result
        str.AppendLine(builder.ToString)
        Return str.ToString
    End Function

    Private Shared Function DisplayErrors(ByVal errors As XmlNodeList) As String
        Dim str As New StringBuilder
        ' Iterate over the list of errors and display error details.
        str.AppendLine("The following errors were encountered while trying to get the headline news:")
        str.AppendLine()
        For Each [error] In errors
            For Each detail In [error].ChildNodes
                str.AppendLine(detail.Name & ": " & detail.InnerText)
            Next detail
            str.AppendLine()
        Next [error]
        Return str.ToString
    End Function
End Class
