﻿Imports System.Text
Imports System.ComponentModel

Public Enum PendingAction
    NoneOutstanding = 0
    PlayAudio
    PlayVideo
    Powerpoint
    Word
    Excel
    ProjectPlan
    Undefined
End Enum

Public Enum ButtonSelected
    NoButton = 0
    Button1
    Button2
    Button3
    Button4
    Button5
    Button6
    Button7
    Button8
End Enum

Public Class ButtonActionEvaluator
    Implements INotifyPropertyChanged

    Private Shared _me As New ButtonActionEvaluator

    Public Shared Function GetButtonActionEvaluator() As ButtonActionEvaluator
        If _me Is Nothing Then
            _me = New ButtonActionEvaluator()
        End If

        Return _me
    End Function

    Private Sub New()
        _actionPending = PendingAction.NoneOutstanding
        _selectedButton = ButtonSelected.NoButton
    End Sub

    Private _actionPending As PendingAction = PendingAction.NoneOutstanding
    Public Property ActionPending As PendingAction
        Get
            Return _actionPending
        End Get

        Set(ByVal value As PendingAction)
            _actionPending = value
            NotifyPropertyChanged("ActionPending")
        End Set
    End Property

    Private _selectedButton As ButtonSelected = ButtonSelected.NoButton
    Public Property SelectedButton As ButtonSelected
        Get
            Return _selectedButton
        End Get

        Set(ByVal value As ButtonSelected)
            _selectedButton = value
            NotifyPropertyChanged("SelectedButton")
        End Set
    End Property

    ''' <summary>
    ''' This method is used to get an integer representation of the Button/Action state - used in conjection with a switch statement to determine appropriate action.
    ''' </summary>
    ''' <returns>[int] representing the state of the Button/Action context.</returns>
    Public Function EvaluateCriteria() As Integer
        If _actionPending = PendingAction.NoneOutstanding OrElse _selectedButton = ButtonSelected.NoButton Then
            Return 0
        End If

        Return Convert.ToInt16((ConvertToBin(CInt(Fix(_selectedButton))) & ConvertToBin(CInt(Fix(_actionPending)))), 2)
    End Function

    Private Shared Function ConvertToBin(ByVal i As Integer) As String
        Dim r As String = Convert.ToString(i, 2)
        Do While r.Length < 4
            r = "0" & r
        Loop
        Return r
    End Function

    Private Sub NotifyPropertyChanged(ByVal info As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(info))
    End Sub

    Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged
End Class
