﻿Imports System.Text
Imports Actions

''' <summary>
''' This class implements the Singleton design pattern (in thread-safe manner) to allow only one LilyContext to exist.
''' The LilyContext is useful in understanding the context that the system is a part of.
''' </summary>
Public Class LilyContext

#Region "Structs"
    ''' <summary>
    ''' This struct is used to contain the data relevant to tracking changing context by the SpeechTracker.
    ''' </summary>
    Public Structure ContextEntry
        Public context As SpeechTracker.Context
        Public timestamp As Date
    End Structure
#End Region

#Region "Class member variables and Properties"
    Private _buttonActionEvaluator As ButtonActionEvaluator
    Public Property ButtonActionEvaluator As ButtonActionEvaluator
        Get
            Return _buttonActionEvaluator
        End Get

        Set(ByVal value As ButtonActionEvaluator)
            _buttonActionEvaluator = value
        End Set
    End Property

    Private _isMoviePlaying As Boolean = False
    Public Property IsMoviePlaying As Boolean
        Get
            Return _isMoviePlaying
        End Get

        Set(ByVal value As Boolean)
            _isMoviePlaying = value
        End Set
    End Property

    Private _isMusicPlaying As Boolean = False
    Public Property IsMusicPlaying As Boolean
        Get
            Return _isMusicPlaying
        End Get

        Set(ByVal value As Boolean)
            _isMusicPlaying = value
        End Set
    End Property

    Private _endUser As Person
    Public Property EndUser As Person
        Get
            Return _endUser
        End Get

        Set(ByVal value As Person)
            _endUser = value
        End Set
    End Property

    Private _currentContext As SpeechTracker.Context
    Public ReadOnly Property CurrentContext As SpeechTracker.Context
        Get
            Return _currentContext
        End Get
    End Property

    Private _contextHistory As List(Of ContextEntry)
    Public ReadOnly Property ContextHistory As List(Of ContextEntry)
        Get
            Return _contextHistory
        End Get
    End Property

    Private sp_trk As SpeechTracker
    Public ReadOnly Property GetSpeechTracker As SpeechTracker
        Get
            Return sp_trk
        End Get
    End Property

    Private Shared _me As New LilyContext
#End Region

#Region "Constructors"
    Private Sub New()
        sp_trk = New SpeechTracker
        AddHandler sp_trk.SpeechContextChanged, AddressOf sp_trk_SpeechContextChanged
        _endUser = New Person
        _currentContext = SpeechTracker.Context.General
        _contextHistory = New List(Of ContextEntry)
        _contextHistory.Add(New ContextEntry With {.context = SpeechTracker.Context.General, .timestamp = Date.Now})
        _buttonActionEvaluator = ButtonActionEvaluator.GetButtonActionEvaluator
        AddHandler _buttonActionEvaluator.PropertyChanged, AddressOf buttonActionEvaluator_PropertyChanged
    End Sub
#End Region

#Region "Private Methods"
    Private Sub buttonActionEvaluator_PropertyChanged(ByVal sender As Object, ByVal e As System.ComponentModel.PropertyChangedEventArgs)
        If MultiModalReactions.ActOnMultiModalInput(_buttonActionEvaluator.EvaluateCriteria) = PendingActionResult.ClearPendingAction Then
            _buttonActionEvaluator.ActionPending = PendingAction.NoneOutstanding
        End If
    End Sub

    Private Sub sp_trk_SpeechContextChanged(ByVal sender As Object, ByVal e As SpeechTracker.SpeechContextChangedArgs)
        _contextHistory.Add(New ContextEntry With {.context = e.context, .timestamp = Date.Now})
        _currentContext = e.context
    End Sub
#End Region

#Region "Public Methods"
    Public Shared Function GetLilyContext() As LilyContext
        If _me Is Nothing Then
            _me = New LilyContext
        End If
        Return _me
    End Function
#End Region

End Class
