﻿Imports System.IO
Imports System.Threading
Imports Microsoft.Research.Kinect.Audio
Imports Microsoft.Speech.AudioFormat
Imports Microsoft.Speech.Recognition
Imports Microsoft.Speech
Imports Actions

Public Class SpeechTracker

#Region "Events"
    Public Class IntentionArgs
        Inherits EventArgs

        Public Property context As Context
        Public Property general As General
        Public Property research As Research
        Public Property office As Office
        Public Property operations As Operations
        Public Property meeting As Meeting
        Public Property translate As Translate
        Public Property entertain As Entertain
        Public Property phrase As String
        Public Property contextOnly As Boolean
    End Class

    Public Class SpeechContextChangedArgs
        Inherits EventArgs

        Public Property context As Context
    End Class

    Public Event SpeechDetected As EventHandler(Of IntentionArgs)
    Public Event SpeechContextChanged As EventHandler(Of SpeechContextChangedArgs)
#End Region

#Region "Enumerations"
    Public Enum Context
        Context = 0
        General
        Research
        Office
        Operations
        Meeting
        Translate
        Entertain
    End Enum

    Public Enum General
        None = 0
        Time
        Log
        Schedule
        Pulse
        Shutdown
        ColorViewer
        DepthViewer
        SkeletonViewer
        Buttons
        Affirmation
        Thanks
        Negative
        TrackOn
        TrackOff
        TranslationView
        NewsView
    End Enum

    Public Enum Research
        None = 0
        Lookup
        Spelling
        News
    End Enum

    Public Enum Office
        None = 0
        Word
        OpenADocument
        Outlook
        Excel
        PowerPoint
        OpenAPresentation
        PPTNextSlide
        PPTPreviousSlide
        StartPresentation
        OpenAProjectPlan
    End Enum

    Public Enum Operations
        None = 0
        SkeletonTracker
        CameraUp
        CameraDown
        CameraCenter
        CourseControl
        FineControl
    End Enum

    Public Enum Meeting
        None = 0
        Snapshot
        CaptureVideo
        CaptureAudio
        [Stop]
    End Enum

    Public Enum Translate
        None = 0
        TurkishEnglish
        EnglishTurkish
        ArabicEnglish
        EnglishArabic
        SpanishEnglish
        EnglishSpanish
    End Enum

    Public Enum Entertain
        None = 0
        Music
        Video
    End Enum
#End Region

#Region "Grammars"
    Private Structure Intention
        Public context As Context
        Public general As General
        Public research As Research
        Public office As Office
        Public kinnyops As Operations
        Public meeting As Meeting
        Public translate As Translate
        Public entertain As Entertain
        Public contextOnly As Boolean
    End Structure

    Private ContextPhrases As New Dictionary(Of String, Intention) From {{"General", New Intention With {.context = Context.General,
                                                                                                           .contextOnly = True}},
                                                                           {"Research", New Intention With {.context = Context.Research,
                                                                                                            .contextOnly = True}},
                                                                           {"I need some information", New Intention With {.context = Context.Research,
                                                                                                                           .contextOnly = True}},
                                                                           {"Lets Research", New Intention With {.context = Context.Research,
                                                                                                                 .contextOnly = True}},
                                                                           {"I need to do some Research", New Intention With {.context = Context.Research,
                                                                                                                              .contextOnly = True}},
                                                                           {"Lily, I need you to do some Research", New Intention With {.context = Context.Research,
                                                                                                                                        .contextOnly = True}},
                                                                           {"Office", New Intention With {.context = Context.Office,
                                                                                                          .contextOnly = True}},
                                                                           {"Kinect", New Intention With {.context = Context.Operations,
                                                                                                          .contextOnly = True}},
                                                                           {"Meeting", New Intention With {.context = Context.Meeting,
                                                                                                           .contextOnly = True}},
                                                                           {"Translate", New Intention With {.context = Context.Translate,
                                                                                                             .contextOnly = True}},
                                                                           {"Lily can you translate for me", New Intention With {.context = Context.Translate,
                                                                                                                                 .contextOnly = True}},
                                                                           {"Lily please translate for me", New Intention With {.context = Context.Translate,
                                                                                                                                .contextOnly = True}},
                                                                           {"Translation", New Intention With {.context = Context.Translate,
                                                                                                               .contextOnly = True}},
                                                                           {"Entertain", New Intention With {.context = Context.Entertain,
                                                                                                             .contextOnly = True}}}

    Private GeneralPhrases As New Dictionary(Of String, Intention) From {{"Time", New Intention With {.context = Context.General,
                                                                                                      .contextOnly = False,
                                                                                                      .general = General.Time}},
                                                                         {"What is the time", New Intention With {.context = Context.General,
                                                                                                                  .contextOnly = False,
                                                                                                                  .general = General.Time}},
                                                                         {"What time is it", New Intention With {.context = Context.General,
                                                                                                                 .contextOnly = False,
                                                                                                                 .general = General.Time}},
                                                                         {"Lily what time is it", New Intention With {.context = Context.General,
                                                                                                                      .contextOnly = False,
                                                                                                                      .general = General.Time}},
                                                                         {"Error log", New Intention With {.context = Context.General,
                                                                                                           .contextOnly = False,
                                                                                                           .general = General.Log}},
                                                                         {"Schedule", New Intention With {.context = Context.General,
                                                                                                          .contextOnly = False,
                                                                                                          .general = General.Schedule}},
                                                                         {"Lily", New Intention With {.context = Context.General,
                                                                                                      .contextOnly = False,
                                                                                                      .general = General.Pulse}},
                                                                         {"Lily are you there", New Intention With {.context = Context.General,
                                                                                                                    .contextOnly = False,
                                                                                                                    .general = General.Pulse}},
                                                                         {"Lily are you listening", New Intention With {.context = Context.General,
                                                                                                                        .contextOnly = False,
                                                                                                                        .general = General.Pulse}},
                                                                         {"Lily please shut down", New Intention With {.context = Context.General,
                                                                                                                       .contextOnly = False,
                                                                                                                       .general = General.Shutdown}},
                                                                         {"Color viewer", New Intention With {.context = Context.General,
                                                                                                              .contextOnly = False,
                                                                                                              .general = General.ColorViewer}},
                                                                         {"Depth viewer", New Intention With {.context = Context.General,
                                                                                                              .contextOnly = False,
                                                                                                              .general = General.DepthViewer}},
                                                                         {"Skeleton viewer", New Intention With {.context = Context.General,
                                                                                                                 .contextOnly = False,
                                                                                                                 .general = General.SkeletonViewer}},
                                                                         {"Lily, show me the color viewer", New Intention With {.context = Context.General,
                                                                                                                                .contextOnly = False,
                                                                                                                                .general = General.ColorViewer}},
                                                                         {"Lily, show me the depth viewer", New Intention With {.context = Context.General,
                                                                                                                                .contextOnly = False,
                                                                                                                                .general = General.DepthViewer}},
                                                                         {"Lily, show me the skeleton viewer", New Intention With {.context = Context.General,
                                                                                                                                   .contextOnly = False,
                                                                                                                                   .general = General.SkeletonViewer}},
                                                                         {"Translation viewer", New Intention With {.context = Context.General,
                                                                                                                    .contextOnly = False,
                                                                                                                    .general = General.TranslationView}},
                                                                         {"News viewer", New Intention With {.context = Context.General,
                                                                                                             .contextOnly = False,
                                                                                                             .general = General.NewsView}},
                                                                         {"Show me the buttons", New Intention With {.context = Context.General,
                                                                                                                     .contextOnly = False,
                                                                                                                     .general = General.Buttons}},
                                                                         {"Lily, buttons please", New Intention With {.context = Context.General,
                                                                                                                      .contextOnly = False,
                                                                                                                      .general = General.Buttons}},
                                                                         {"Buttons please", New Intention With {.context = Context.General,
                                                                                                                .contextOnly = False,
                                                                                                                .general = General.Buttons}},
                                                                         {"Lily, show me the buttons", New Intention With {.context = Context.General,
                                                                                                                           .contextOnly = False,
                                                                                                                           .general = General.Buttons}},
                                                                         {"Lily, can you show me the buttons", New Intention With {.context = Context.General,
                                                                                                                                   .contextOnly = False,
                                                                                                                                   .general = General.Buttons}},
                                                                         {"Lily, good job", New Intention With {.context = Context.General,
                                                                                                                .contextOnly = False,
                                                                                                                .general = General.Affirmation}},
                                                                         {"good job Lily", New Intention With {.context = Context.General,
                                                                                                               .contextOnly = False,
                                                                                                               .general = General.Affirmation}},
                                                                         {"awesome Lily", New Intention With {.context = Context.General,
                                                                                                              .contextOnly = False,
                                                                                                              .general = General.Affirmation}},
                                                                         {"awesome job Lily", New Intention With {.context = Context.General,
                                                                                                                  .contextOnly = False,
                                                                                                                  .general = General.Affirmation}},
                                                                         {"Thank you Lily", New Intention With {.context = Context.General,
                                                                                                                .contextOnly = False, .general = General.Thanks}},
                                                                         {"Thanks Lily", New Intention With {.context = Context.General,
                                                                                                             .contextOnly = False,
                                                                                                             .general = General.Thanks}},
                                                                         {"No Lily", New Intention With {.context = Context.General,
                                                                                                         .contextOnly = False,
                                                                                                         .general = General.Negative}},
                                                                         {"Thats not right Lily", New Intention With {.context = Context.General,
                                                                                                                      .contextOnly = False,
                                                                                                                      .general = General.Negative}},
                                                                         {"Lily thats not right", New Intention With {.context = Context.General,
                                                                                                                      .contextOnly = False,
                                                                                                                      .general = General.Negative}},
                                                                         {"Thats not what I meant Lily", New Intention With {.context = Context.General,
                                                                                                                             .contextOnly = False,
                                                                                                                             .general = General.Negative}},
                                                                         {"That sucks Lily", New Intention With {.context = Context.General,
                                                                                                                 .contextOnly = False,
                                                                                                                 .general = General.Negative}},
                                                                         {"Lily track off", New Intention With {.context = Context.General,
                                                                                                                .contextOnly = False,
                                                                                                                .general = General.TrackOff}},
                                                                         {"Lily please stop tracking", New Intention With {.context = Context.General,
                                                                                                                           .contextOnly = False,
                                                                                                                           .general = General.TrackOff}},
                                                                         {"Lily turn tracking off", New Intention With {.context = Context.General,
                                                                                                                        .contextOnly = False,
                                                                                                                        .general = General.TrackOff}},
                                                                         {"Lily please turn tracking off", New Intention With {.context = Context.General,
                                                                                                                               .contextOnly = False,
                                                                                                                               .general = General.TrackOff}},
                                                                         {"Lily track on", New Intention With {.context = Context.General,
                                                                                                               .contextOnly = False,
                                                                                                               .general = General.TrackOn}},
                                                                         {"Lily please start tracking", New Intention With {.context = Context.General,
                                                                                                                            .contextOnly = False,
                                                                                                                            .general = General.TrackOn}},
                                                                         {"Lily turn tracking on", New Intention With {.context = Context.General,
                                                                                                                       .contextOnly = False,
                                                                                                                       .general = General.TrackOn}},
                                                                         {"Lily please turn tracking on", New Intention With {.context = Context.General,
                                                                                                                              .contextOnly = False,
                                                                                                                              .general = General.TrackOn}}}

    Private ResearchPhrases As New Dictionary(Of String, Intention) From {{"Look up", New Intention With {.context = Context.Research,
                                                                                                            .contextOnly = False,
                                                                                                            .research = Research.Lookup}},
                                                                            {"Bing", New Intention With {.context = Context.Research,
                                                                                                         .contextOnly = False,
                                                                                                         .research = Research.Lookup}},
                                                                            {"Google", New Intention With {.context = Context.Research,
                                                                                                           .contextOnly = False,
                                                                                                           .research = Research.Lookup}},
                                                                            {"Search for", New Intention With {.context = Context.Research,
                                                                                                               .contextOnly = False,
                                                                                                               .research = Research.Lookup}},
                                                                            {"Spell", New Intention With {.context = Context.Research,
                                                                                                          .contextOnly = False,
                                                                                                          .research = Research.Spelling}},
                                                                            {"Checking the spelling of", New Intention With {.context = Context.Research,
                                                                                                                             .contextOnly = False,
                                                                                                                             .research = Research.Spelling}},
                                                                            {"How do you spell", New Intention With {.context = Context.Research,
                                                                                                                     .contextOnly = False,
                                                                                                                     .research = Research.Spelling}},
                                                                            {"News", New Intention With {.context = Context.Research,
                                                                                                         .contextOnly = False,
                                                                                                         .research = Research.News}},
                                                                            {"What's in the headlines", New Intention With {.context = Context.Research,
                                                                                                                            .contextOnly = False,
                                                                                                                            .research = Research.News}},
                                                                            {"What are the headlines", New Intention With {.context = Context.Research,
                                                                                                                           .contextOnly = False,
                                                                                                                           .research = Research.News}},
                                                                            {"What's in the news", New Intention With {.context = Context.Research,
                                                                                                                       .contextOnly = False,
                                                                                                                       .research = Research.News}},
                                                                            {"Lily what's in the news", New Intention With {.context = Context.Research,
                                                                                                                            .contextOnly = False,
                                                                                                                            .research = Research.News}},
                                                                            {"What's making the news", New Intention With {.context = Context.Research,
                                                                                                                           .contextOnly = False,
                                                                                                                           .research = Research.News}},
                                                                            {"Can you show me the news", New Intention With {.context = Context.Research,
                                                                                                                             .contextOnly = False,
                                                                                                                             .research = Research.News}},
                                                                            {"Lily please show me the news", New Intention With {.context = Context.Research,
                                                                                                                                 .contextOnly = False,
                                                                                                                                 .research = Research.News}}}

    Private OfficePhrases As New Dictionary(Of String, Intention) From {{"Excel", New Intention With {.context = Context.Office,
                                                                                                      .contextOnly = False,
                                                                                                      .office = Office.Excel}},
                                                                        {"Lets crunch some numbers", New Intention With {.context = Context.Office,
                                                                                                                         .contextOnly = False,
                                                                                                                         .office = Office.Excel}},
                                                                        {"Start a spreadsheet", New Intention With {.context = Context.Office,
                                                                                                                    .contextOnly = False,
                                                                                                                    .office = Office.Excel}},
                                                                        {"Create a spreadsheet", New Intention With {.context = Context.Office,
                                                                                                                     .contextOnly = False,
                                                                                                                     .office = Office.Excel}},
                                                                        {"Open Excel", New Intention With {.context = Context.Office,
                                                                                                           .contextOnly = False,
                                                                                                           .office = Office.Excel}},
                                                                        {"Outlook", New Intention With {.context = Context.Office,
                                                                                                        .contextOnly = False,
                                                                                                        .office = Office.Outlook}},
                                                                        {"Open Outlook", New Intention With {.context = Context.Office,
                                                                                                             .contextOnly = False,
                                                                                                             .office = Office.Outlook}},
                                                                        {"Check email", New Intention With {.context = Context.Office,
                                                                                                            .contextOnly = False,
                                                                                                            .office = Office.Outlook}},
                                                                        {"Lets look at email", New Intention With {.context = Context.Office,
                                                                                                                   .contextOnly = False,
                                                                                                                   .office = Office.Outlook}},
                                                                        {"Show me my email", New Intention With {.context = Context.Office,
                                                                                                                 .contextOnly = False,
                                                                                                                 .office = Office.Outlook}},
                                                                        {"Show me Outlook", New Intention With {.context = Context.Office,
                                                                                                                .contextOnly = False,
                                                                                                                .office = Office.Outlook}},
                                                                        {"Power point", New Intention With {.context = Context.Office,
                                                                                                            .contextOnly = False,
                                                                                                            .office = Office.PowerPoint}},
                                                                        {"Open Power point", New Intention With {.context = Context.Office,
                                                                                                                 .contextOnly = False,
                                                                                                                 .office = Office.PowerPoint}},
                                                                        {"Open the presentation", New Intention With {.context = Context.Office,
                                                                                                                      .contextOnly = False,
                                                                                                                      .office = Office.OpenAPresentation}},
                                                                        {"Create a Power point", New Intention With {.context = Context.Office,
                                                                                                                     .contextOnly = False,
                                                                                                                     .office = Office.PowerPoint}},
                                                                        {"Create a presentation", New Intention With {.context = Context.Office,
                                                                                                                      .contextOnly = False,
                                                                                                                      .office = Office.PowerPoint}},
                                                                        {"Create a brief", New Intention With {.context = Context.Office,
                                                                                                               .contextOnly = False,
                                                                                                               .office = Office.PowerPoint}},
                                                                        {"Next slide please", New Intention With {.context = Context.Office,
                                                                                                                  .contextOnly = False,
                                                                                                                  .office = Office.PPTNextSlide}},
                                                                        {"Previous slide please", New Intention With {.context = Context.Office,
                                                                                                                      .contextOnly = False,
                                                                                                                      .office = Office.PPTPreviousSlide}},
                                                                        {"Start the presentation", New Intention With {.context = Context.Office,
                                                                                                                       .contextOnly = False,
                                                                                                                       .office = Office.StartPresentation}},
                                                                        {"Word", New Intention With {.context = Context.Office,
                                                                                                     .contextOnly = False,
                                                                                                     .office = Office.Word}},
                                                                        {"Open Word", New Intention With {.context = Context.Office,
                                                                                                          .contextOnly = False,
                                                                                                          .office = Office.Word}},
                                                                        {"Create a document", New Intention With {.context = Context.Office,
                                                                                                                  .contextOnly = False,
                                                                                                                  .office = Office.Word}},
                                                                        {"Create a new document", New Intention With {.context = Context.Office,
                                                                                                                      .contextOnly = False,
                                                                                                                      .office = Office.Word}},
                                                                        {"Start a document", New Intention With {.context = Context.Office,
                                                                                                                 .contextOnly = False,
                                                                                                                 .office = Office.Word}},
                                                                        {"Start a new document", New Intention With {.context = Context.Office,
                                                                                                                     .contextOnly = False,
                                                                                                                     .office = Office.Word}},
                                                                        {"Open the document", New Intention With {.context = Context.Office,
                                                                                                                  .contextOnly = False,
                                                                                                                  .office = Office.OpenADocument}},
                                                                        {"Open the project plan", New Intention With {.context = Context.Office,
                                                                                                                      .contextOnly = False,
                                                                                                                      .office = Office.OpenAProjectPlan}}}

    Private OperationsPhrases As New Dictionary(Of String, Intention) From {{"Skeleton tracker", New Intention With {.context = Context.Operations,
                                                                                                                     .contextOnly = False,
                                                                                                                     .kinnyops = Operations.SkeletonTracker}},
                                                                            {"Camera up", New Intention With {.context = Context.Operations,
                                                                                                              .contextOnly = False,
                                                                                                              .kinnyops = Operations.CameraUp}},
                                                                            {"Lily please tilt the camera up", New Intention With {.context = Context.Operations,
                                                                                                                                   .contextOnly = False,
                                                                                                                                   .kinnyops = Operations.CameraUp}},
                                                                            {"Camera down", New Intention With {.context = Context.Operations,
                                                                                                                .contextOnly = False,
                                                                                                                .kinnyops = Operations.CameraDown}},
                                                                            {"Lily please tilt the camera down", New Intention With {.context = Context.Operations,
                                                                                                                                     .contextOnly = False,
                                                                                                                                     .kinnyops = Operations.CameraDown}},
                                                                            {"Camera center", New Intention With {.context = Context.Operations,
                                                                                                                  .contextOnly = False,
                                                                                                                  .kinnyops = Operations.CameraCenter}},
                                                                            {"Lily please center the camera", New Intention With {.context = Context.Operations,
                                                                                                                                  .contextOnly = False,
                                                                                                                                  .kinnyops = Operations.CameraCenter}},
                                                                            {"course control", New Intention With {.context = Context.Operations,
                                                                                                                   .contextOnly = False,
                                                                                                                   .kinnyops = Operations.CourseControl}},
                                                                            {"fine control", New Intention With {.context = Context.Operations,
                                                                                                                 .contextOnly = False,
                                                                                                                 .kinnyops = Operations.FineControl}}}

    Private MeetingPhrases As New Dictionary(Of String, Intention) From {{"Record this", New Intention With {.context = Context.Meeting,
                                                                                                             .contextOnly = False,
                                                                                                             .meeting = Meeting.CaptureAudio}},
                                                                         {"Video this", New Intention With {.context = Context.Meeting,
                                                                                                            .contextOnly = False,
                                                                                                            .meeting = Meeting.CaptureVideo}},
                                                                         {"Stop recording", New Intention With {.context = Context.Meeting,
                                                                                                                .contextOnly = False,
                                                                                                                .meeting = Meeting.Stop}},
                                                                         {"Lily snapshot", New Intention With {.context = Context.Meeting,
                                                                                                               .contextOnly = False,
                                                                                                               .meeting = Meeting.Snapshot}},
                                                                         {"Lily please take a snapshot", New Intention With {.context = Context.Meeting,
                                                                                                                             .contextOnly = False,
                                                                                                                             .meeting = Meeting.Snapshot}},
                                                                         {"Lily take a snapshot", New Intention With {.context = Context.Meeting, .contextOnly = False, .meeting = Meeting.Snapshot}}}

    Private TranslationPhrases As New Dictionary(Of String, Intention) From {{"Arabic to English", New Intention With {.context = Context.Translate,
                                                                                                                       .contextOnly = False,
                                                                                                                       .translate = Translate.ArabicEnglish}},
                                                                             {"English to Arabic", New Intention With {.context = Context.Translate,
                                                                                                                       .contextOnly = False,
                                                                                                                       .translate = Translate.EnglishArabic}},
                                                                             {"English to Spanish", New Intention With {.context = Context.Translate,
                                                                                                                        .contextOnly = False,
                                                                                                                        .translate = Translate.EnglishSpanish}},
                                                                             {"English to Turkish", New Intention With {.context = Context.Translate,
                                                                                                                        .contextOnly = False,
                                                                                                                        .translate = Translate.EnglishTurkish}},
                                                                             {"Spanish to English", New Intention With {.context = Context.Translate,
                                                                                                                        .contextOnly = False,
                                                                                                                        .translate = Translate.SpanishEnglish}},
                                                                             {"Turkish to English", New Intention With {.context = Context.Translate,
                                                                                                                        .contextOnly = False,
                                                                                                                        .translate = Translate.TurkishEnglish}}}

    Private EntertainPhrases As New Dictionary(Of String, Intention) From {{"I want to listen to music", New Intention With {.context = Context.Entertain,
                                                                                                                            .contextOnly = False,
                                                                                                                            .entertain = Entertain.Music}},
                                                                           {"Can we listen to music", New Intention With {.context = Context.Entertain,
                                                                                                                          .contextOnly = False,
                                                                                                                          .entertain = Entertain.Music}},
                                                                           {"Lily, music please", New Intention With {.context = Context.Entertain,
                                                                                                                      .contextOnly = False,
                                                                                                                      .entertain = Entertain.Music}},
                                                                           {"I need some music", New Intention With {.context = Context.Entertain,
                                                                                                                     .contextOnly = False,
                                                                                                                     .entertain = Entertain.Music}},
                                                                           {"I want to watch a movie", New Intention With {.context = Context.Entertain,
                                                                                                                           .contextOnly = False,
                                                                                                                           .entertain = Entertain.Video}},
                                                                           {"Lily, movie please", New Intention With {.context = Context.Entertain,
                                                                                                                      .contextOnly = False,
                                                                                                                      .entertain = Entertain.Video}},
                                                                           {"Lets watch a movie", New Intention With {.context = Context.Entertain,
                                                                                                                      .contextOnly = False,
                                                                                                                      .entertain = Entertain.Video}},
                                                                           {"Can we watch a movie", New Intention With {.context = Context.Entertain,
                                                                                                                        .contextOnly = False,
                                                                                                                        .entertain = Entertain.Video}}}
#End Region

#Region "Class member variables"
    Private kinectSource As KinectAudioSource
    Private ri As RecognizerInfo
    Private sre As SpeechRecognitionEngine
    Private Const RecognizerId As String = "SR_MS_en-US_Kinect_10.0"
    Private _keepListening As Boolean = True
    Private allDicts As List(Of Dictionary(Of String, Intention))
    Private gContext As Grammar
    Private gGeneral As Grammar
    Private gResearch As Grammar
    Private gOffice As Grammar
    Private gOperations As Grammar
    Private gMeeting As Grammar
    Private gTranslation As Grammar
    Private gEntertain As Grammar
#End Region

#Region "Constructors"
    Public Sub New()
        ri = SpeechRecognitionEngine.InstalledRecognizers.Where(Function(r) r.Id = RecognizerId).FirstOrDefault
        If ri Is Nothing Then
            ' no RecognizerInfo => bail
            Exit Sub
        End If
        sre = New SpeechRecognitionEngine(ri.Id)
        ' build our categories of Choices
        Dim contextPhrases = New Choices
        For Each phrase In Me.ContextPhrases
            contextPhrases.Add(phrase.Key)
        Next phrase

        Dim generalPhrases = New Choices
        For Each phrase In Me.GeneralPhrases
            generalPhrases.Add(phrase.Key)
        Next phrase

        Dim researchPhrases = New Choices
        For Each phrase In Me.ResearchPhrases
            researchPhrases.Add(phrase.Key)
        Next phrase

        Dim officePhrases = New Choices
        For Each phrase In Me.OfficePhrases
            officePhrases.Add(phrase.Key)
        Next phrase

        Dim operationsPhrases = New Choices
        For Each phrase In Me.OperationsPhrases
            operationsPhrases.Add(phrase.Key)
        Next phrase

        Dim meetingsPhrases = New Choices
        For Each phrase In MeetingPhrases
            meetingsPhrases.Add(phrase.Key)
        Next phrase

        Dim translationPhrases = New Choices
        For Each phrase In Me.TranslationPhrases
            translationPhrases.Add(phrase.Key)
        Next phrase

        Dim entertainPhrases = New Choices
        For Each phrase In Me.EntertainPhrases
            entertainPhrases.Add(phrase.Key)
        Next phrase

        'create our GrammarBuilders
        Dim gbContext = New GrammarBuilder
        gbContext.Append(contextPhrases)

        Dim gbGeneral = New GrammarBuilder
        gbGeneral.Append(generalPhrases)

        Dim gbResearch = New GrammarBuilder
        gbResearch.Append(researchPhrases)

        Dim gbOffice = New GrammarBuilder
        gbOffice.Append(officePhrases)

        Dim gbOperations = New GrammarBuilder
        gbOperations.Append(operationsPhrases)

        Dim gbMeeting = New GrammarBuilder
        gbMeeting.Append(meetingsPhrases)

        Dim gbTranslation = New GrammarBuilder
        gbTranslation.Append(translationPhrases)

        Dim gbEntertain = New GrammarBuilder
        gbEntertain.Append(entertainPhrases)

        'and finally create our Grammars
        gContext = New Grammar(gbContext)
        gGeneral = New Grammar(gbGeneral)
        gResearch = New Grammar(gbResearch)
        gOffice = New Grammar(gbOffice)
        gOperations = New Grammar(gbOperations)
        gMeeting = New Grammar(gbMeeting)
        gTranslation = New Grammar(gbTranslation)
        gEntertain = New Grammar(gbEntertain)

        'we are only going to load the Context and General grammars at this point
        sre.LoadGrammar(gContext)
        sre.LoadGrammar(gGeneral)

        allDicts = New List(Of Dictionary(Of String, Intention))() From {Me.ContextPhrases, Me.GeneralPhrases, Me.ResearchPhrases, Me.OfficePhrases, Me.OperationsPhrases, MeetingPhrases, Me.TranslationPhrases, Me.EntertainPhrases}

        AddHandler sre.SpeechRecognized, AddressOf sre_SpeechRecognized
        AddHandler sre.SpeechHypothesized, AddressOf sre_SpeechHypothesized
        AddHandler sre.SpeechRecognitionRejected, AddressOf sre_SpeechRecognitionRejected

        Dim t = New Thread(AddressOf BeginListening)
        t.Priority = ThreadPriority.Highest
        t.Start()
    End Sub

    Private Sub sre_SpeechRecognitionRejected(ByVal sender As Object, ByVal e As SpeechRecognitionRejectedEventArgs)
        'Throw New NotImplementedException
    End Sub

    Private Sub sre_SpeechHypothesized(ByVal sender As Object, ByVal e As SpeechHypothesizedEventArgs)
        'Throw New NotImplementedException
    End Sub

    Private Sub sre_SpeechRecognized(ByVal sender As Object, ByVal e As SpeechRecognizedEventArgs)
        If e.Result.Confidence < 0.95F Then
            Exit Sub
        End If

        If SpeechDetectedEvent Is Nothing Then
            Exit Sub
        End If

        Dim intent = New IntentionArgs

        Dim found As Boolean = False
        Dim i As Integer = 0
        Do While i < allDicts.Count AndAlso Not found
            For Each phrase In allDicts(i)
                If e.Result.Text.Equals(phrase.Key) Then
                    'we have a match from our grammars => process the audio command
                    intent.context = phrase.Value.context
                    intent.entertain = phrase.Value.entertain
                    intent.general = phrase.Value.general
                    intent.operations = phrase.Value.kinnyops
                    intent.meeting = phrase.Value.meeting
                    intent.office = phrase.Value.office
                    intent.research = phrase.Value.research
                    intent.translate = phrase.Value.translate
                    intent.contextOnly = phrase.Value.contextOnly
                    intent.phrase = e.Result.Text
                    found = True
                    Exit For
                End If
            Next phrase
            i += 1
        Loop
        If Not found Then
            ' the speech detected was not matched => continue listening
            Exit Sub
        End If
        If intent.contextOnly Then
            'this command is only switching context => we do not need to raise the SpeechDetected event; swap out grammars based on context
            'however, we want to throw the SpeechContextChanged event if it is being listened to
            If SpeechContextChangedEvent IsNot Nothing Then
                Dim arg As New SpeechContextChangedArgs
                arg.context = intent.context
                RaiseEvent SpeechContextChanged(Me, arg)
            End If
            Select Case intent.context
                Case Context.Entertain
                    ClearGrammars()
                    sre.LoadGrammar(gEntertain)
                Case Context.Operations
                    ClearGrammars()
                    sre.LoadGrammar(gOperations)
                Case Context.Meeting
                    ClearGrammars()
                    sre.LoadGrammar(gMeeting)
                Case Context.Office
                    ClearGrammars()
                    sre.LoadGrammar(gOffice)
                Case Context.Research
                    ClearGrammars()
                    sre.LoadGrammar(gResearch)
                Case Context.Translate
                    GeneralActions.SpeakAsync("Yes. I can translate for you. Tell me the languages you want translated from, and to.")
                    ClearGrammars()
                    sre.LoadGrammar(gTranslation)
                Case Else
            End Select
        Else
            RaiseEvent SpeechDetected(Me, intent)
        End If
    End Sub

    Private Sub ClearGrammars()
        Dim remove As New List(Of Grammar)
        For Each g In sre.Grammars
            If Not (g.Equals(gContext) OrElse g.Equals(gGeneral)) Then
                remove.Add(g)
            End If
        Next g
        For Each g In remove
            sre.UnloadGrammar(g)
        Next g
    End Sub
#End Region

#Region "Private methods"
    Private Sub BeginListening()
        kinectSource = New KinectAudioSource
        kinectSource.SystemMode = SystemMode.OptibeamArrayOnly
        kinectSource.FeatureMode = True
        kinectSource.AutomaticGainControl = True
        kinectSource.NoiseSuppression = True
        kinectSource.MicArrayMode = MicArrayMode.MicArrayAdaptiveBeam
        Dim kinectStream = kinectSource.Start
        sre.SetInputToAudioStream(kinectStream, New SpeechAudioFormatInfo(EncodingFormat.Pcm, 16000, 16, 1, 32000, 2, Nothing))
        sre.RecognizeAsync(RecognizeMode.Multiple)
    End Sub

    Private Sub StopListening()
        If sre IsNot Nothing Then
            sre.RecognizeAsyncCancel()
            sre.RecognizeAsyncStop()
            kinectSource.Dispose()
        End If
    End Sub
#End Region

#Region "Protected methods"

#End Region

#Region "Public methods"

#End Region

#Region "Properties"
    Public Property KeepListening As Boolean
        Get
            Return _keepListening
        End Get

        Set(ByVal value As Boolean)
            _keepListening = value
            If Not _keepListening Then
                ' it is no longer desired for Lily to be listening => 
                StopListening()
            End If
        End Set
    End Property
#End Region

End Class
